from src.config.datamodels import ScanTaskBaseConfig, ScanTaskRuntimeConfig, ScanTaskMetaData
from src.interfaces.datarequests import ScanTaskData, ReadTaskStaticConfigFromFile, BaseDataRequest, RunTask
from src.interfaces.task import Task, TaskRunTime, ReadTaskConfigData
from src.interfaces.requests import IInspectionConfig, IRequestParameters, IConfidenceConfig, IInfoTypeMeta, \
    IScanForLongTexts, IMeta, IInfoType, IGetInfoTypeRequestParameters, IConfidenceThresholdForSomeParams, \
    INonCountrySpecificConfig, IExclusionConfigKeyedByInfotype
from src.types.tasks import TaskType, TaskConfigs
from src.utils.s3_helper import getS3Data, putS3Data, convertToCSV



from typing import List, Literal, Optional

from pydantic import BaseModel

DetectionSensitivity = Literal['high', 'low']


class IInfoTypeMeta(BaseModel):
    domains: Optional[List[str]] = None


class IInfoType(BaseModel):
    name: str
    meta: Optional[IInfoTypeMeta] = None


class IScanForLongTexts(BaseModel):
    columnNumbers: Optional[List[int]] = None
    columnNames: Optional[List[str]] = None


class IGetInfoTypeRequestParameters(BaseModel):
    type: Literal['allGeneric', 'allCountry', 'all']


class IRequestParameters(BaseModel):
    snapshot: Optional[bool] = None
    detailed: Optional[bool] = None
    detectionSensitivity: Optional[DetectionSensitivity] = None
    getConfidenceDetailed: Optional[bool] = None
    predictNullColumns: Optional[bool] = None
    allExcept: Optional[str] = None  ## json string (legacy)
    scanRegexOnly: Optional[bool] = None
    scanWithoutRegexBoundaries: Optional[bool] = None
    showNeighbourMatch: Optional[bool] = None
    showNeighbourhood: Optional[int] = None
    maskDetails: Optional[bool] = None
    sample: Optional[bool] = None
    sampleSize: Optional[str] = None
    interruptIfTooLong: Optional[bool] = None
    interruptTimeBudgetPerMB: Optional[int] = None


class INonCountrySpecificConfig(BaseModel):
    toScan: bool
    nonCountrySpecificExcept: Optional[List[str]] = None  # which generic infotypes to not include
    countrySpecificInclude: Optional[List[str]] = None  # which country-specific infotypes to include


suffixForConfidenceType = Literal['defaultsOnly', 'all', 'none']


class IConfidenceThresholdForSomeParams(BaseModel):
    confidenceThreshold: int
    infoTypes: List[str]


class IConfidenceConfig(BaseModel):
    useDefaultPossibles: bool
    confidenceThreshold: Optional[int] = None  # replaces the default confidence threshold
    confidenceThresholdForSomeInfoTypes: Optional[IConfidenceThresholdForSomeParams] = None
    suffixForConfidence: Optional[suffixForConfidenceType] = None


class IInspectionConfig(BaseModel):
    infoTypes: List[IInfoType]
    neighbourRequirePII: Optional[List[str]] = None
    infoTypesWithoutMasking: Optional[List[str]] = None
    confidenceConfig: Optional[IConfidenceConfig] = None
    allNonCountrySpecific: Optional[INonCountrySpecificConfig] = None
    allExcept: Optional[List[str]] = None
    exclusions: Optional[List[str]] = None
    scanForLongTexts: Optional[IScanForLongTexts] = None


class IExclusionConfigKeyedByInfotype(BaseModel):
    infoType: List[str]


class mimeDict(BaseModel):
    headers: Optional[str] = None
    delimiter: Optional[str] = None
    quoteChar: Optional[str] = None
    excludeColumns: Optional[List[int]] = None


class IMeta(BaseModel):
    mimetype: Optional[str] = None
    mimetypeMeta: Optional[mimeDict] = None
    fileName: Optional[str] = None
    entityName: Optional[str] = None



from typing import Dict, Any, Optional
from pydantic import BaseModel


class BaseDataRequest(BaseModel):
    success: bool
    error: Optional[str] = None
    data: Optional[Dict] = None
    infotypes: Optional[Dict] = None


ReadTaskStaticConfigFromFile: BaseDataRequest


class ScanTaskData(BaseDataRequest):
    data: Optional[Any] = None
    success: bool


class RunTask(BaseDataRequest):
    scanTaskID: str



from pydantic.dataclasses import dataclass

from src.types.tasks import TaskType, TaskConfigs
from src.config.datamodels import ScanTaskBaseConfig, ScanTaskRuntimeConfig, ScanTaskMetaData
from src.interfaces.datarequests import ScanTaskData
from typing import Dict, Any, Optional
from pydantic import BaseModel
from abc import ABC, abstractmethod


class ReadTaskConfigData(BaseModel):
    data: Optional[Dict] = None


@dataclass
class Task(ABC):
    taskType: TaskType
    taskConfig: TaskConfigs
    ScanTaskBaseConfig: Optional[Any] = None

    @abstractmethod
    def readTaskStaticConfigData(self) -> ReadTaskConfigData:
        raise NotImplementedError


@dataclass
class TaskRunTime(ABC):
    task: Task

    @abstractmethod
    def readTaskStaticConfigData(self, dataReadTaskConfig: Any) -> ReadTaskConfigData:
        raise NotImplementedError

    @abstractmethod
    def readTaskData(self, dataReadTask: Any) -> ScanTaskData:
        raise NotImplementedError

    @abstractmethod
    def writeTaskData(self, dataWriteTask: Any) -> ScanTaskData:
        raise NotImplementedError

    @abstractmethod
    def runTask(self, dataTask: Any) -> ScanTaskMetaData:
        raise NotImplementedError



from typing import Dict, Any, Optional, List
from pydantic import validate_arguments, BaseModel
from pydantic.dataclasses import dataclass
from src.interfaces.requests import IInspectionConfig, IRequestParameters


class ScanTaskBaseConfig(BaseModel):
    taskConfiguration: Optional[Any] = None
    taskIOConfiguration: Optional[Any] = None


class ScanTaskRuntimeConfig(BaseModel):
    runTaskID: str
    runTaskTracker: Dict
    runTaskDetails: Dict


class Data(BaseModel):
    file: str
    basic: Optional[List[str]]
    advanced: Optional[List[str]]


class Resp_Data(BaseModel):
    snapshot: Optional[str]
    detailed: Optional[str]
    error: Optional[str]


class Response(BaseModel):
    data: Optional[Resp_Data] = None
    success: bool


class ScanTaskReqDet(BaseModel):
    inspectionConfig: IInspectionConfig
    data: Data
    query: IRequestParameters
    response: Response


class ScanTaskMetaData(BaseModel):
    scanTaskID: str
    scanTaskRequestDetails: ScanTaskReqDet



from enum import Enum


class TaskType(Enum):
    INSPECTION_ML = 'IN_ML'
    INSPECTION_IMAGE = 'IN_IMG'
    INSPECTION_NER = 'IN_NER'
    INSPECTION_CLASSIFY = 'IN_CLS'
    TEST = 'TEST'


class TaskConfigs(Enum):
    INSPECTION_ML = './config/ib.yml'
    INSPECTION_IMAGE = './config/ia.yml'
    INSPECTION_NER = './config/test.yml'
    INSPECTION_CLASSIFY = './config/test.yml'
    TEST = './config/test.yml'



from distutils.core import setup

setup(
    name='python_temporal',
    version='0.0.7',
    author='X.X',
    author_email='xx@example.com',
    url='http://www.example.com/',
    license='LICENSE',
    packages=['src', 'src.config', 'src.interfaces', 'src.types', 'src.utils'],
    description="The description of the package",
    long_description_content_type="text/markdown",
    long_description="test",
    install_requires=['temporal-python-sdk',
                      'boto3~=1.24.22',
                      'PyYAML',
                      'pandas~=1.4.3',
                      'numpy',
                      'pydantic~=1.9.1'
                      ]
)



import boto3
import pandas as pd
from boto3.s3.transfer import TransferConfig


async def getS3Data(params: any):
    s3 = boto3.resource("s3")

    try:
        obj = s3.Object(params.Bucket, params.Key)
        data_content = obj.get()['Body'].read()

        print({"data": params.Key, "success": True})

        return {
            "file_key": obj.key,
            "file_body": data_content.decode('utf-8'),
            "file_length": obj.content_length}

    except OSError as e:
        print(e)


async def putS3Data(params: any):
    s3 = boto3.resource('s3')
    config = TransferConfig(multipart_threshold=1024 * params.partsize, max_concurrency=params.concurrency,
                            multipart_chunksize=1024 * params.chunksize, use_threads=params.threads)
    try:
        s3.Object(params.bucket_name, params.bucket_filename).upload_file(params.key,
                                                                          Config=config
                                                                          )
        print({"data": params.Key, "success": True})
    except OSError as e:
        print(e)

    return None


async def convertToCSV(params: any):
    with open(params, encoding='utf-8') as inputfile:
        df = pd.read_json(inputfile)

    return await df
